/**
 * ContentsCluster - a Solaris cluster and the files it contains
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;

/*
 * While this class looks and feels like ContentsPackage, it is implemented
 * very differently. The ContentsPackage can be constructed directly from
 * parsing the contents file. A ContentsCluster is a virtual object made
 * by amalgamating all the packages from a cluster.
 */

public class ContentsCluster {

    /*
     * For a package the contents file itself guarantees uniqueness of the
     * files, but when contructing the list of files for a cluster we need
     * to handle this ourselves.
     */
    private Set <ContentsFileDetail> fileset;

    public ContentsCluster(PackageCluster pc) {
	fileset = new HashSet <ContentsFileDetail> ();
	ContentsParser cp = ContentsParser.getInstance();
	for (SolarisPackage pkg : pc.getPackages()) {
	    ContentsPackage cpp = cp.getPackage(pkg.getName());
	    if (cpp != null) {
		fileset.addAll(cpp.getDetails());
	    }
	}
    }

    public ContentsCluster(MetaCluster mc) {
	fileset = new HashSet <ContentsFileDetail> ();
	ContentsParser cp = ContentsParser.getInstance();
	for (PackageCluster pc : mc.getClusters()) {
	    for (SolarisPackage pkg : pc.getPackages()) {
		ContentsPackage cpp = cp.getPackage(pkg.getName());
		if (cpp != null) {
		    fileset.addAll(cpp.getDetails());
		}
	    }
	}
	for (SolarisPackage pkg : mc.getPackages()) {
	    ContentsPackage cpp = cp.getPackage(pkg.getName());
	    if (cpp != null) {
		fileset.addAll(cpp.getDetails());
	    }
	}
    }

    public int numEntries() {
	return fileset.size();
    }

    public int numDirectories() {
	int i = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.isDirectory()) {
		i++;
	    }
	}
	return i;
    }

    /*
     * pkginfo -l reports "linked files" just for hard links
     */
    public int numHardLinks() {
	int i = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.ftype.equals("l")) {
		i++;
	    }
	}
	return i;
    }

    public int numSymLinks() {
	int i = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.ftype.equals("s")) {
		i++;
	    }
	}
	return i;
    }

    /*
     * Devices are files of type b or c
     */
    public int numDevices() {
	int i = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.ftype.equals("c") || cfd.ftype.equals("b")) {
		i++;
	    }
	}
	return i;
    }

    /*
     * Shared files are in more than one package
     */
    public int numShared() {
	int i = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.isShared()) {
		i++;
	    }
	}
	return i;
    }

    /*
     * Add up the space used
     */
    public long spaceUsed() {
	long l = 0;
	for (ContentsFileDetail cfd : fileset) {
	    if (cfd.ftype.equals("f") || cfd.ftype.equals("v")
			|| cfd.ftype.equals("e")) {
		l += Long.parseLong(cfd.size);
	    }
	}
	return l;
    }

    /*
     * Match the ContentsPackage getDetails();
     */
    public List <ContentsFileDetail> getDetails() {
	return new ArrayList <ContentsFileDetail> (fileset);
    }
}
