/**
 * PackageCluster - describe a Solaris install cluster
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;

/**
 * Describes a Solaris install cluster
 *
 * @author Peter Tribble
 * @version 1.0
 *
 */
public class PackageCluster {

    public static final int FULLY_INSTALLED = 0;
    public static final int PARTIALLY_INSTALLED = 1;
    public static final int NOT_INSTALLED = 2;

    private String cname;
    private String name;
    private String description;
    private List <SolarisPackage> packages;

    private boolean selected = false;
    private boolean mustbeselected = false;

    public PackageCluster(String cname) {
	this.cname = cname;
	packages = new ArrayList <SolarisPackage> ();
    }

    public void setName(String name) {
	this.name = name;
    }

    public void setDescription(String description) {
	this.description = description;
    }

    public String getName() {
	return name;
    }

    public String getDescription() {
	return description;
    }

    public String toString() {
	return cname;
    }

    public String getClusterName() {
	return cname;
    }

    public void addPackage(SolarisPackage p) {
	packages.add(p);
    }

    // FIXME should it be a Set?
    public List <SolarisPackage> getPackages() {
	return packages;
    }

    public boolean containsPackage(SolarisPackage p) {
	if (packages.contains(p)) {
	    return true;
	}
	// check if any names match
	return containsPackage(p.getName());
    }

    public boolean containsPackage(String pname) {
	// check if any names match
	for (SolarisPackage pkg : packages) {
	    if (pkg.getName().equals(pname)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Accumulate the dependencies for this cluster
     */
    public Map <String, String> dependMap() {
	Map <String, String> m = new HashMap <String, String> ();
	for (SolarisPackage pkg : packages) {
	    Map <String, String> m2 = pkg.dependMap();
	    for (String pname : m2.keySet()) {
		m.put(pname, m2.get(pname));
	    }
	}
	// remove any of our own packages
	for (SolarisPackage pkg : packages) {
	    m.remove(pkg.getName());
	}
	return m;
    }

    /**
     * Return the Set of package names that his cluster declares itself
     * dependent on.
     */
    public Set <String> getDependencySet() {
	Set <String> deps = new HashSet <String> ();
	for (SolarisPackage pkg : packages) {
	    deps.addAll(pkg.getDependencySet());
	}
	// remove any of our own packages
	for (SolarisPackage pkg : packages) {
	    deps.remove(pkg.getName());
	}
	return deps;
    }

    /**
     * Return the Set of packages that are dependant on this cluster.
     */
    public Set <SolarisPackage> getDependantSet() {
	Set <SolarisPackage> depSet = new HashSet <SolarisPackage> ();
	for (SolarisPackage pkg : packages) {
	    Set <SolarisPackage> tset = pkg.getDependantSet();
	    if (tset != null) {
		depSet.addAll(tset);
	    }
	}
	// remove any of our own packages
	depSet.removeAll(packages);
	return depSet;
    }

    /**
     * Return the Set of packages that are dependant on this cluster
     * and are selected.
     */
    public Set <SolarisPackage> getSelectedDependantSet() {
	Set <SolarisPackage> depSet = new HashSet <SolarisPackage> ();
	for (SolarisPackage pkg : getDependantSet()) {
	    if (pkg.getSelected()) {
		depSet.add(pkg);
	    }
	}
	return depSet;
    }

    /**
     * Shows whether this cluster is fully installed, partially installed,
     * or not installed.
     */
    public int installedStatus() {
	int ipkg = 0;
	int iinst = 0;
	for (SolarisPackage pkg : packages) {
	    ipkg++;
	    if (pkg.exists()) {
		iinst++;
	    }
	}
	if (iinst == 0) {
	    return NOT_INSTALLED;
	}
	if (iinst == ipkg) {
	    return FULLY_INSTALLED;
	}
	return PARTIALLY_INSTALLED;
    }

    /**
     * Shows whether this cluster is fully installed, partially installed,
     * or not installed.
     */
    public String installedStatusText() {
	String t;
	switch (installedStatus()) {
	    case FULLY_INSTALLED:
		t = "Fully Installed";
		break;
	    case PARTIALLY_INSTALLED:
		t = "Partially Installed";
		break;
	    case NOT_INSTALLED:
		t = "Not Installed";
		break;
	    default:
		t = "Unknown";
	}
	return t;
    }

    /**
     * Shows whether this cluster is fully selected, partially selected,
     * or not selected, based on whether its constituent packages are
     * selected.
     */
    public int selectedStatus() {
	int ipkg = 0;
	int iinst = 0;
	for (SolarisPackage pkg : packages) {
	    ipkg++;
	    if (pkg.getSelected()) {
		iinst++;
	    }
	}
	if (iinst == 0) {
	    return NOT_INSTALLED;
	}
	if (iinst == ipkg) {
	    return FULLY_INSTALLED;
	}
	return PARTIALLY_INSTALLED;
    }

    /**
     * Sets whether this package cluster is selected. Propagates that selection
     * to packages contained in this cluster.
     */
    public void setSelected(boolean selected) {
	this.selected = selected;
	for (SolarisPackage pkg : packages) {
	    pkg.setSelected(selected);
	}
    }

    /**
     * Gets whether this package cluster is selected.
     */
    public boolean getSelected() {
	return selected;
    }

    /**
     * Sets whether this package cluster must be selected. Propagates that
     * selection to packages contained in this cluster.
     */
    public void forceSelected() {
	mustbeselected = true;
	for (SolarisPackage pkg : packages) {
	    pkg.forceSelected();
	}
    }

    /**
     * Gets whether this package cluster must be selected. Packages and
     * clusters in the SUNWCmreq metacluster must be selected.
     */
    public boolean mustBeSelected() {
	return mustbeselected;
    }

    /**
     * Gets whether this package cluster contains one or more, but not all,
     * packages that must be selected. Packages and clusters in the SUNWCmreq
     * metacluster must be selected.
     */
    public boolean mustBePartiallySelected() {
	/*
	 * The aim here is to identify clusters that have some but not all
	 * packages that must be selected. Therefore, if the whole cluster
	 * must be selected, then it can't be partially selected, so we
	 * return false here.
	 */
	if (mustbeselected) {
	    return false;
	}
	for (SolarisPackage pkg : packages) {
	    if (pkg.mustBeSelected()) {
		return true;
	    }
	}
	return false;
    }
}
